> {
> module Calc where
> import Data.Char
> import Data.List (sort)
> import Debug.Trace
> import Language.Haskell.Ranges
> }

First thing to declare is the name of your parser,
and the type of the tokens the parser reads.

> %name calc
> %tokentype { Token }

The parser will be of type [Token] -> ?, where ? is determined by the
production rules.  Now we declare all the possible tokens:

> %token 
>	let		{ TokenLet }
>	in		{ TokenIn }
>	int		{ TokenInt $$ }
>	var		{ TokenVar $$ }
>	'='		{ TokenEq }
>	'+'		{ TokenPlus }
>	'-'		{ TokenMinus }
>	'*'		{ TokenTimes }
>	'/'		{ TokenDiv }
>	'('		{ TokenOB }
>	')'		{ TokenCB }

The left hand side are the names of the terminals or tokens,
and the right hand side is how to pattern match them.

Like yacc, we include %% here, for no real reason.

> %%

Now we have the production rules.

let r x = Ranges [(x, Fin x)] in calc [(TokenMinus, r 1), (TokenMinus, r 2), (TokenPlus, r 3)]

> Exp :: { Expr }
> Exp : '-' altBlock { $2 }
> altBlock : '(' ZERO(alts) ')' { Block $2 }
>          | INDENT(altLayout) { Layout $1 }
> altLayout : { [] }
>  | FIRST_EQ(alts) altLayout { $1 : $2 }
> alts :
>   alt { [$1] }
> | alt '/' alts { $1 : $3 }
> alt : '+' { Nothing }
>  | Exp { Just $1 }

> {-
> Exp :: { Exp }
> Exp : -- INDENT(Factor) FIRST_EQ(Epsilon) '+' { Exp1 (Term (Factor $1)) }
>  -- | INDENT(Factor) Epsilon '+' { Exp1 (Term (Factor $1)) }
>  -- | FIRST_EQ(INDENT(Exp)) Epsilon '-' { $1 }
>  -- | Factor Factor { Exp1 (Term (Factor $1)) }
>   Epsilon ZERO(Factor) { Exp1 (Term (Factor $1)) }
> -- let var '=' INDENT(INDENT(Exp)) in Exp	{ Let $2 $4 $6 }
> --    | INDENT(Exp1)			{ Exp1 $1 }
> --    | ZERO(Term) { Exp1 (Term $1) }
> 
> {-Exp1 : Exp1 '+' Term		{ Plus $1 $3 }
>      | Exp1 '-' Term		{ Minus $1 $3 }
>      | Term			{ Term $1 }
> 
> Term : Term '*' Factor	{ Times $1 $3 }
>      | Term '/' Factor	{ Div $1 $3 }
>      | Factor			{ Factor $1 } -}
> Epsilon : { () }
> 
> Factor : int			{ Int $1 }
> --       | Factor '+' Factor {Int $1}
> --	 | var			{ Var $1 }
> --	 | '(' Exp ')'		{ Brack $2 }
> -}

We are simply returning the parsed data structure !
Now we need some extra code, to support this parser,
and make in complete:

> {

All parsers must declair this function, 
which is called when an error is detected.
Note that currently we do no error recovery.

> happyError :: [(Token, Ranges)] -> a
> happyError _ = error ("Parse error\n")

Now we declare the datastructure that we are parsing.

> data Expr = Block [Maybe Expr] | Layout [[Maybe Expr]] deriving (Show)
> data Exp  = Let String Exp Exp | Exp1 Exp1 deriving (Show)
> data Exp1 = Plus Exp1 Term | Minus Exp1 Term | Term Term  deriving (Show)
> data Term = Times Term Factor | Div Term Factor | Factor Factor  deriving (Show)
> data Factor = Int Int | Var String | Brack Exp  deriving (Show)

The datastructure for the tokens...

> data Token
>	= TokenLet
>	| TokenIn
>	| TokenInt Int
>	| TokenVar String
>	| TokenEq
>	| TokenPlus
>	| TokenMinus
>	| TokenTimes
>	| TokenDiv
>	| TokenOB
>	| TokenCB

.. and a simple lexer that returns this datastructure.

> lexer :: String -> [Token]
> lexer [] = []
> lexer (c:cs) 
>	| isSpace c = lexer cs
> 	| isAlpha c = lexVar (c:cs)
>	| isDigit c = lexNum (c:cs)
> lexer ('=':cs) = TokenEq : lexer cs
> lexer ('+':cs) = TokenPlus : lexer cs
> lexer ('-':cs) = TokenMinus : lexer cs
> lexer ('*':cs) = TokenTimes : lexer cs
> lexer ('/':cs) = TokenDiv : lexer cs
> lexer ('(':cs) = TokenOB : lexer cs
> lexer (')':cs) = TokenCB : lexer cs

> lexNum cs = TokenInt (read num) : lexer rest
>	where (num,rest) = span isDigit cs

> lexVar cs =
>    case span isAlpha cs of
>	("let",rest) -> TokenLet : lexer rest
>	("in",rest)  -> TokenIn : lexer rest
>	(var,rest)   -> TokenVar var : lexer rest

To run the program, call this in gofer, or use some code
to print it.

 runCalc :: String -> Exp
 runCalc = calc . lexer

Here we test our parser.

 main = case runCalc "1 + 2 + 3" of {
	(Exp1 (Plus (Plus (Term (Factor (Int 1))) (Factor (Int 2))) (Factor (Int 3))))  ->
 	case runCalc "1 * 2 + 3" of {
	(Exp1 (Plus (Term (Times (Factor (Int 1)) (Int 2))) (Factor (Int 3)))) ->
	case runCalc "1 + 2 * 3" of {
	(Exp1 (Plus (Term (Factor (Int 1))) (Times (Factor (Int 2)) (Int 3)))) ->
	case runCalc "let x = 2 in x * (x - 2)" of {
	(Let "x" (Exp1 (Term (Factor (Int 2)))) (Exp1 (Term (Times (Factor (Var "x")) (Brack (Exp1 (Minus (Term (Factor (Var "x"))) (Factor (Int 2))))))))) -> print "AndysTest works\n" ; 
	_ -> quit } ; _ -> quit } ; _ -> quit } ; _ -> quit }
 quit = print "runCalc failed\n"

> }
